--
-- Manfredas Zabarauskas
-- University of Edinburgh
-- BSc Artificial Intelligence & Computer Science '08
--
-- Informatics 1A Competition Entry
--
-- Fractal.hs
--

module Fractal (                
                generateJulia,      -- Complex Double -> Int -> Int -> Color -> [RasterPoint]                
                fractalBackground,  -- [RasterPoint]
                cloud, dust         -- Complex Double
               )
       where

import Complex

import Render

fractalBackground :: [RasterPoint]
fractalBackground = generateJulia dust width (height - 200) (1,1,5)

-- Julia Set Generation Algorithm
generateJulia :: Complex Double -> -- initial value (starting point on complex plane)
                 Int ->            -- width
                 Int ->            -- height
                 Color ->          -- color
                 [RasterPoint]     -- resulting fractal
generateJulia c width height (r,g,b) = [((x, y), calcPoint x y) | x <- [0..width - 1], y <- [0..height - 1]]
    where
      calcPoint :: Int -> Int -> Color
      calcPoint x y = (clr*r, clr*g, clr*b)
          where 
            clr = julia' (re :+ im) 0

            re = (3.0 * fromIntegral x / fromIntegral width) - 1.5
            im = (3.0 * fromIntegral y / fromIntegral height) - 1.5
                      
            julia' :: Complex Double -> Int -> Int
            julia' z iter
              | iter > 64 = 0
              | otherwise = if (magnitude z' > 2) then iter
                            else julia' z' (iter+1)
                                where z' = z^2 + c

-- starting values
cloud :: Complex Double
cloud = (-0.835) :+ (-0.2321)

dust :: Complex Double
dust = (-0.423) :+ (0.745)
