--
-- Manfredas Zabarauskas
-- University of Edinburgh
-- BSc Artificial Intelligence & Computer Science '08
--
-- Informatics 1A Competition Entry
--
-- Graphics.hs
--

module Graphics (
                 Point,
                 (.-.),        -- Point -> Point -> Point
                 (.+.),        -- Point -> Point -> Point
                 get_X,        -- Point -> Coord
                 get_Y,        -- Point -> Coord
                 get_Z,        -- Point -> Coord

                 Shape (..),    
                 zSort,        -- [Shape] -> [Shape]
                 translate,    -- Shape -> Point -> Shape
                 drawShapes    -- [Shape] -> Bitmap -> Bitmap
                ) 
       where

import List

import Render

type Point = (Coord, Coord, Coord)

data Shape = Line Point Point Color
           | Triangle Point Point Point Color
           | Rectangle Point Point Point Point Color
           | Polygon [Point] Color
           deriving Show

get_X :: Point -> Coord
get_X (x, y, z) = x

get_Y :: Point -> Coord
get_Y (x, y, z) = y

get_Z :: Point -> Coord
get_Z (x, y, z) = z

(.+.) :: Point -> Point -> Point
(x1, y1, z1) .+. (x2, y2, z2) = (x1+x2, y1+y2, z1+z2)

(.-.) :: Point -> Point -> Point
(x1, y1, z1) .-. (x2, y2, z2) = (x1-x2, y1-y2, z1-z2)

{- toVector :: Point -> Vector
toVector (x, y, z) = Vec3 x y z

fromVector :: Vector -> Point
fromVector (Vec3 x y z) = (x, y, z)

getNormal :: Shape -> Vector
getNormal (Triangle a b c clr) = ((toVector a) <-> (toVector b)) <*> ((toVector c) <-> (toVector b))
getNormal (Rectangle a b c d clr) = getNormal (Triangle a b c clr) -}

translate :: Shape -> Point -> Shape
translate (Line a b clr) p = Line (a .+. p) (b .+. p) clr
translate (Triangle a b c clr) p = Triangle (a .+. p) (b .+. p) (c .+. p) clr
translate (Rectangle a b c d clr) p = Rectangle (a .+. p) (b .+. p) (c .+. p) (d .+. p) clr
translate (Polygon pts clr) p = Polygon (map (p .+.) pts) clr

{- backfaceCull :: Shape -> Bool
backfaceCull (Rectangle a b c d clr) = ((getNormal (Rectangle a b c d clr)) <.> (view_vector <-> toVector a)) < 0
-}

-- Z-Sort (Painter`s) Algorithm
zSort :: [Shape] -> [Shape]
zSort = sortBy isFurther
        where
          isFurther shape1 shape2
              | (z_sum shape1) > (z_sum shape2) = LT
              | otherwise = GT

          z_sum (Polygon pts clr) = calc_sum pts
          z_sum (Rectangle a b c d clr) = calc_sum [a, b, c, d]
          z_sum (Triangle a b c clr) = calc_sum [a, b, c]
          z_sum (Line a b clr) = calc_sum [a, b]
          
          calc_sum pts = fromIntegral ((sum . (map get_Z)) pts) / fromIntegral (length pts)


drawShapes :: [Shape] -> Bitmap -> Bitmap
drawShapes = renderPointsToBMP . concat . (map toRasterPoints)

draw :: Shape -> Bitmap -> Bitmap
draw = renderPointsToBMP . toRasterPoints

toRasterPoints :: Shape -> [RasterPoint]
toRasterPoints (Line a b clr) = rasterLine (projectPoint a clr) 
                                                   (projectPoint b clr)

toRasterPoints (Triangle a b c clr) = (rasterLine proj_a proj_b) ++ 
                                      (rasterLine proj_b proj_c) ++ 
                                      (rasterLine proj_c proj_a)
    where proj_a = projectPoint a clr
          proj_b = projectPoint b clr
          proj_c = projectPoint c clr

toRasterPoints (Rectangle a b c d clr) = (rasterLine proj_a proj_b) ++ 
                                         (rasterLine proj_b proj_c) ++ 
                                         (rasterLine proj_c proj_d) ++ 
                                         (rasterLine proj_d proj_a)
    where proj_a = projectPoint a clr
          proj_b = projectPoint b clr
          proj_c = projectPoint c clr
          proj_d = projectPoint d clr

toRasterPoints (Polygon pts clr) = (concat . map (uncurry rasterLine)) (zip proj_pts (tail proj_pts ++ [head proj_pts]))
    where proj_pts = map (\x ->  projectPoint x clr) pts