--
-- Manfredas Zabarauskas
-- University of Edinburgh
-- BSc Artificial Intelligence & Computer Science '08
--
-- Informatics 1A Competition Entry
--
-- Main.hs
--
-- Instructions: use GHC to compile and run, i.e.
-- > ghc --make Main
-- > ./Main (Linux) or main.exe (Windows)
--
-- Output: DCIM0001.ppm
--
-- Note: Expected rendering time - ~1 min.
-- 

import Random
import Array
import System
import Data.Time.Clock
import Data.Time.Calendar

import Render
import Graphics
import World
import Fractal
import Text
import PPM

init_rect = Rectangle (0, 0, size) (size, 0, size) (size, 0, 0) (0, 0, 0) dark_blue
            where size = 4*width

main :: IO ()
main = do
  generator <- newStdGen
  time <- getCurrentTime

  let rand_l :: [Int]
      rand_l = randomRs (-200, 400) generator
     
      rand_f :: [Float]
      rand_f = randomRs (0, 1) generator

      terrain = generateTerrain init_rect rand_l 6    -- generating terrain
      mountains = getMountains terrain 60             -- getting mountain list
      trees = generateTrees mountains                 -- generating recursive trees
      podium = generatePodium terrain 300 (rand_l!!0) -- generating podium
      trees_n_podium = zSort $ (concat . (map getShapeList)) trees ++ podium -- z-sorting trees and podium
      lambda_obj = placeLambda podium                 -- placing lambda

      final_bitmap = draw_border $                    -- generating output bitmap
                     renderPointsToBMP (stringToText label grey (25, 20)) $ 
                     superimposePointsToBMP (placeCloud (lambda_obj!!lambda_top_index) 50 800 800) $ 
                     drawShapes (getShapeList terrain ++ trees_n_podium ++ lambda_obj) $ 
                     renderPointsToBMP (fractalBackground ++ (generateStars 150 rand_f)) init_bitmap
                     
                     where draw_border = renderPointsToBMP (rasterLine ((0, 0), white) ((0, height-1), white) ++
                                                            rasterLine ((width-1, 0), white) ((width-1, height-1), white) ++
                                                            rasterLine ((0, 0), white) ((width-1, 0), white) ++
                                                            rasterLine ((0, height-1), white) ((width-1, height-1), white))
                           
                           label = "1 NIGHT IN HASKELL                  " ++ (show day) ++ "/" ++ (show month) ++ "/" ++ year
                           (year_4, month, day) =  (toGregorian . utctDay) time
                           year = if (year_2 < 10) then ("0" ++ (show year_2)) else (show year_2) where year_2 = year_4 `mod` 100

  writeFile "DCIM0001.ppm" (convertToPPM (expandBitmap final_bitmap 50)) -- writing output file
